<?php
// ----------------------------------------------------------------------------------
// Constants
// ----------------------------------------------------------------------------------
// Version                   : 0.7
// Authors                   : Chris Bizer (chris@bizer.de),
// Daniel Westphal (dawe@gmx.de),
// Leandro Mariano Lopez (llopez@xinergiaargentina.com),
// Radoslaw Oldakowski (radol@gmx.de)

// Description               : Constants and default configuration
// ----------------------------------------------------------------------------------
// History:
// 11-18-200                  : RDF_PARSE_TYPE_COLLECTION, RDF_XMLLITERAL,
// IN_PROPERTY_PARSE_TYPE_COLLECTION, VALIDATE_IDS added.
// 11-13-2003                 : HIDE_ADVERTISE added
// 11-12-2003                 : FIX_BLANKNODES added
// 07-27-2003                 : Database, RDQL Error Messages,
// RDQL default namespace prefixes added
// 02-12-2003                 : XML_NAMESPACE_DECLARATION_PREFIX changed.
// 01-15-2003                 : Some syntax corrections to avoid PHP notices added.
// 01-10-2003                 : Constants RDF_NODEID, RDF_SEEALSO, RDF_OBJECT_TYPE_BNODE,
// RDF_SUBJECT_TYPE_BNODE added
// 12-18-2002                 : RDF_DATATYPE, $short_datatype added
// 10-03-2002                 : Some RDF Shema constants added
// 09-15-2002                 : Initial version
// ----------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------
// General
// ----------------------------------------------------------------------------------
define('RDFAPI_ERROR', 'RDFAPI error ');
define('DEFAULT_ALGORITHM', 'MD5');
define('DEFAULT_ENCODING', 'UTF-8');
define('INDENTATION', '    ');
define('LINEFEED', chr(10));
// ----------------------------------------------------------------------------------
// Model
// ----------------------------------------------------------------------------------
define('BNODE_PREFIX', 'bNode');
// ----------------------------------------------------------------------------------
// Parser
// ----------------------------------------------------------------------------------
define('VALIDATE_IDS', true);
define('FIX_BLANKNODES', false);
define('NAMESPACE_SEPARATOR_CHAR', '^');
define('NAMESPACE_SEPARATOR_STRING', '^');
define('IN_TOP_LEVEL', 0);
define('IN_RDF', 1);
define('IN_DESCRIPTION', 2);
define('IN_PROPERTY_UNKNOWN_OBJECT', 3);
define('IN_PROPERTY_RESOURCE', 4);
define('IN_PROPERTY_EMPTY_RESOURCE', 5);
define('IN_PROPERTY_LITERAL', 6);
define('IN_PROPERTY_PARSE_TYPE_LITERAL', 7);
define('IN_PROPERTY_PARSE_TYPE_RESOURCE', 8);
define('IN_XML', 9);
define('IN_UNKNOWN', 10);
define('IN_PROPERTY_PARSE_TYPE_COLLECTION', 11);
define('RDF_SUBJECT_TYPE_URI', 0);
define('RDF_SUBJECT_TYPE_DISTRIBUTED', 1);
define('RDF_SUBJECT_TYPE_PREFIX', 2);
define('RDF_SUBJECT_TYPE_ANONYMOUS', 3);
define('RDF_SUBJECT_TYPE_BNODE', 4);
define('RDF_OBJECT_TYPE_RESOURCE', 0);
define('RDF_OBJECT_TYPE_LITERAL', 1);
define('RDF_OBJECT_TYPE_XML', 2);
define('RDF_OBJECT_TYPE_BNODE', 3);
// ----------------------------------------------------------------------------------
// Serializer
// ----------------------------------------------------------------------------------
define('HIDE_ADVERTISE', false);
define('SER_USE_ENTITIES', false);
define('SER_USE_ATTRIBUTES', false);
define('SER_SORT_MODEL', true);
define('SER_RDF_QNAMES', true);
define('SER_XML_DECLARATION', true);

define('GENERAL_PREFIX_BASE', 'ns');
define('MAX_ALLOWED_ABBREVIATED_LENGTH', 60);
define('USE_ANY_QUOTE', 0);
define('USE_CDATA', 1);
// ----------------------------------------------------------------------------------
// Util
// ----------------------------------------------------------------------------------
define('HTML_TABLE_HEADER_COLOR', '#FFFFFF');
define('HTML_TABLE_RESOURCE_COLOR', '#FFFFCC');
define('HTML_TABLE_LITERAL_COLOR', '#E7E7EF');
define('HTML_TABLE_BNODE_COLOR', '#FFCCFF');
define('HTML_TABLE_RDF_NS_COLOR', '#CCFFCC');
// ----------------------------------------------------------------------------------
// RDF
// ----------------------------------------------------------------------------------
define('RDF_NAMESPACE_URI', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#');
define('RDF_NAMESPACE_PREFIX', 'rdf');
define('RDF_RDF', 'RDF');
define('RDF_DESCRIPTION', 'Description');
define('RDF_ID', 'ID');
define('RDF_ABOUT', 'about');
define('RDF_ABOUT_EACH', 'aboutEach');
define('RDF_ABOUT_EACH_PREFIX', 'aboutEachPrefix');
define('RDF_BAG_ID', 'bagID');
define('RDF_RESOURCE', 'resource');
define('RDF_VALUE', 'value');
define('RDF_PARSE_TYPE', 'parseType');
define('RDF_PARSE_TYPE_LITERAL', 'Literal');
define('RDF_PARSE_TYPE_RESOURCE', 'Resource');
define('RDF_PARSE_TYPE_COLLECTION', 'Collection');
define('RDF_TYPE', 'type');
define('RDF_BAG', 'Bag');
define('RDF_SEQ', 'Seq');
define('RDF_ALT', 'Alt');
define('RDF_LI', 'li');
define('RDF_STATEMENT', 'Statement');
define('RDF_SUBJECT', 'subject');
define('RDF_PREDICATE', 'predicate');
define('RDF_OBJECT', 'object');
define('RDF_NODEID', 'nodeID');
define('RDF_DATATYPE', 'datatype');
define('RDF_SEEALSO', 'seeAlso');
define('RDF_PROPERTY', 'Property');
define('RDF_LIST', 'List');
define('RDF_NIL', 'nil');
define('RDF_REST', 'rest');
define('RDF_FIRST', 'first');
define('RDF_XMLLITERAL', 'XMLLiteral');
// ----------------------------------------------------------------------------------
// RDF Schema
// ----------------------------------------------------------------------------------
define('RDF_SCHEMA_URI', 'http://www.w3.org/2000/01/rdf-schema#');
define('RDF_DATATYPE_SCHEMA_URI', 'http://www.w3.org/TR/xmlschema-2');
define('RDF_SCHEMA_PREFIX', 'rdfs');
define('RDFS_SUBCLASSOF', 'subClassOf');
define('RDFS_SUBPROPERTYOF', 'subPropertyOf');
// ----------------------------------------------------------------------------------
// XML
// ----------------------------------------------------------------------------------
define('XML_NAMESPACE_PREFIX', 'xml');
define('XML_NAMESPACE_DECLARATION_PREFIX', 'xmlns');
define('XML_NAMESPACE_URI', 'http://www.w3.org/XML/1998/namespace');
define('XML_LANG', 'lang');
define('DATATYPE_SHORTCUT_PREFIX', 'datatype:');
// ----------------------------------------------------------------------------------
// RDF DATATYPE SHORTCUTS (extends datatype shortcuts to the full XML datatype URIs)
// ----------------------------------------------------------------------------------
$short_datatype = array(
    'STRING'   => RDF_DATATYPE_SCHEMA_URI . '#string',
    'DECIMAL'  => RDF_DATATYPE_SCHEMA_URI . '#decimal',
    'INTEGER'  => RDF_DATATYPE_SCHEMA_URI . '#integer',
    'INT'      => RDF_DATATYPE_SCHEMA_URI . '#int',
    'SHORT'    => RDF_DATATYPE_SCHEMA_URI . '#short',
    'BYTE'     => RDF_DATATYPE_SCHEMA_URI . '#byte',
    'LONG'     => RDF_DATATYPE_SCHEMA_URI . '#long',
    'LANGUAGE' => RDF_DATATYPE_SCHEMA_URI . '#language',
    'NAME'     => RDF_DATATYPE_SCHEMA_URI . '#name'
    );
// ----------------------------------------------------------------------------------
// RDQL Error Messages
// ----------------------------------------------------------------------------------
define('RDQL_ERR', 'RDQL error ');
define('RDQL_SYN_ERR', 'RDQL syntax error ');
define('RDQL_SEL_ERR', RDQL_ERR . 'in the SELECT clause: ');
define('RDQL_SRC_ERR', RDQL_ERR . 'in the SOURCE clause: ');
define('RDQL_WHR_ERR', RDQL_ERR . 'in the WHERE clause: ');
define('RDQL_AND_ERR', RDQL_ERR . 'in the AND clause: ');
define('RDQL_USG_ERR', RDQL_ERR . 'in the USING clause: ');
// ----------------------------------------------------------------------------------
// RDQL default namespace prefixes
// ----------------------------------------------------------------------------------
$default_prefixes = array(
    'rdf'  => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
    'rdfs' => 'http://www.w3.org/2000/01/rdf-schema#',
    'xsd'  => 'http://www.w3.org/2001/XMLSchema#'
    );
?>